SOAP.logical <-
function(x)
{
  val <- as.logical(x)

  if(is.na(val)) {
   val <- as.logical(as.integer(x))
  }

  val
}  

SOAPPrimitiveConverters <-
                list("xsd:timeInstant" = as.POSIXct,
                     "xsd:float" = as.numeric,
                     "xsd:int" = as.integer,
                     "xsd:float" = as.numeric,
                     "xsd:string" = as.character,
                     "xsd:boolean" = SOAP.logical,
# Extensions from Datatypes schema                     
                     "xsd:decimal" = as.numeric)


getNodeById <-
  #
  # Find the top-level node identified by id=value of id
  # Strip the `#' prefix off `id' first.
function(id, root)
{
  id <- gsub("^#", "", id)

  for(i in xmlChildren(root)) {
    k <- xmlAttrs(i)
    if(!is.null(k) && !is.na(match("id", names(k))))
      if(id == k[["id"]])
        return(i)
  }

  return(NULL)
}


# Also need general converters that work on nodes
fromSOAP <-
  #
  # Top-level entry point to convert a top-level SOAP XML node to 
  # 
function(node, root = NULL, converters = SOAPPrimitiveConverters, append = TRUE)
{
  if(!missing(converters) && append) {
    SOAPPrimitiveConverters[names(converters)] <- converters
    converters <- SOAPPrimitiveConverters
  }
  
  if(is.character(node)) {
    root <- parseSOAP(node)
    if(xmlName(root) == "Body")
      node <- root[[1]]
    else
      node <- root
  }
  
  type <- NULL
  a <- xmlAttrs(node)
  
  if(!is.null(a)) {
    if(!is.na(match("null", names(a))))
      return(NULL)

    if(!is.na(match("type", names(a))))
      type <- a[["type"]]

    if(!is.na(match("href", names(a)))) {
      n <- getNodeById(a[["href"]], root)
      if(is.null(n))
        stop("Can't find element ", a[["href"]])
      return(fromSOAP(n, root = root, converters = converters))
    }
  }

  if(xmlName(node) == "Array" || (!is.null(type) && type == "SOAP-ENC:Array")) {
    return(fromSOAPArray(node, root = root, converters = converters))
  } else if(xmlSize(node) > 1) {
    return(fromSOAPStruct(node, root = root, converters = converters))
  } 

  if(is.null(type)) {
     # Want to check namespace is SOAP-ENC
     # and ideally don't want to prefix it with xsd:
     # but probably the most usual case is to get the typ
     # as xsd:type from the attribute and so that is the
     # default.
    type <- paste("xsd", xmlName(node), sep=":")
  }
  
  which <- match(type, names(converters))
  if(!is.na(which))
    val <- converters[[which]](xmlValue(node))
  else {
    val <- xmlValue(node)
    warning("Don't understand the SOAP type `", type, "' yet")
  }

  val
}


fromSOAPArray <-
  # Need to handle the partial arrays
  # where individual elements are  specified.
  #
function(node, type = NULL, root = NULL, converters = SOAPPrimitiveConverters)
{
  len <- xmlSize(node)
  a <- xmlAttrs(node)  

  if(missing(type)) {
    type <- a[["arrayType"]]
    if(!is.null(type)) {
      els <- strsplit(type, ":")[[1]]
      type <- gsub("\(.*\)\\\[.*", "\\1", els[2])
      len <- as.integer(gsub(".*\\\[\([0-9,]+\)\\\]", "\\1", els[2]))
      cat("Type:", type, " & length = ", len, "\n")
    }
  }


   # This is the general mechanism for dealing with offsets.
   # It doesn't work in S-Plus since it uses lexical scoping.
   # Need an OOP object for that.
  offset <- 1 
  if(!is.na(match("offset", names(a)))) {
    tmp <- gsub("\\\[\([0-9]+\)\\\]", "\\1", a[["offset"]])
    offset <- as.integer(tmp)
  }

  ans <- vector("list", len)
  val <- xmlApply(node, function(x, type = NULL, root = NULL, converters = SOAPPrimitiveConverters) {
                           z <- fromSOAP(x, root = root, converters = converters)

                           a <- xmlAttrs(x)
                           if(!is.null(a) && !is.na(match("position", names(a)))) {
                             offset <- as.integer(a[["position"]]) + 1
                           }                           

                           ans[[offset]] <<- z
                            # Do we need to differentiate between this coming from
                            # the position or the global offset.
                            # In the case of position, each item will provide 
                            # its own position value.
                           offset <<- offset + 1
                           z
                        }, type=type, converters = converters, root = root)

  return(ans)
}

fromSOAPStruct <-
function(node, root = root, converters = SOAPPrimitiveConverters)
{

   # See if there is a type="value" attribute which we will use
   # for the class.
  a <- xmlAttrs(node)  
  if(!is.null(a) && !is.na(match("type", names(a)))) {
    typeName <- a[["type"]]
  } else
    typeName <- xmlName(node)

  # Now lookup the converters to see if there is an appropriate
  # handler for this type.

  # Otherwise, just use the default mechanism.
  val <- xmlApply(node, fromSOAP, root = root, converters = converters)

  class(val) <- gsub("^[a-zA-Z]+:", "", typeName)
  
  val
}  
