\name{.SOAP}
\alias{.SOAP}
\title{Invoke a SOAP method}
\description{
  This is used to call a SOAP method
  in a SOAP server, passing the relevant arguments
  from S and converting the response into an S object.
  The communication between S and the SOAP server is
  handled via connections.
}
\usage{
.SOAP(server, method, ..., action, nameSpaces = SOAPNameSpaces,
      xmlns = NULL, handlers = .SOAPHandlers(), debug = FALSE)
}
\arguments{
  \item{server}{a \code{SOAPServer} object}
  \item{method}{the name of the SOAP method to invoke}
  \item{\dots}{name=value arguments to pass to the }
  \item{action}{the SOAPAction string to put in the HTTP header. This is
    required.}
  \item{nameSpaces}{a named character vector giving the
    XML namespaces to add to the Body. These are given
    as a named character vector with the names giving the
    local namespace identifier and the value being the
    URI corresponding to that namespace identifier.}
  \item{debug}{a logical value indicating whether we
    want to simply look at the content that would be sent
    to the HTTP/SOAP server. }
  \item{xmlns}{the name space to use for the XML nodes
    which specify the actual method call, i.e. within the BODY.
    This is either a single string, or a name-value pair
    given as a character vector. The name is the namespace identifier
    and the value is the URI.}
  \item{handlers}{a collection of functions that, if present, are called
    at different points in the SOAP invocation to
    process the input and output.
    These can be thought of as event callbacks and include
    \code{action} for creating the final form of the SOAPAction string,
    \code{converter} for processing the XML returned by the SOAP server
    in the case of a successful invocation, and so on.
  }
}
\details{

}
\value{
 An S object representing the return value from the SOAP method invocation.
}
\references{
  \url{http://www.w3.org/TR/SOAP/}
  \url{http://www.omegahat.org/SSOAP},
  \url{http://www.omegahat.org/bugs}.
}
\author{Duncan Temple Lang <duncan@research.bell-labs.com>}
\seealso{
  \code{\link{writeSOAPMessage}}
  \code{\link{isHTTPError}}
  \code{\link{getHTTPContent}}
}

\examples{
\dontrun{
.SOAP(SOAPServer("services.xmethods.net", "soap"),
       "getRate", country1="England", country2 = "Japan",
       action="urn:xmethods-CurrencyExchange")

.SOAP(SOAPServer("services.xmethods.net", "soap/servlet/rpcrouter"),
      "getPrice", "0596000278",
      action="urn:xmethods-BNPriceCheck")

s <- SOAPServer("http://services.xmethods.net/soap")
.SOAP(s,
      "getQuote", "AMZN",
      action="urn:xmethods-delayed-quotes#getQuote")


.SOAP(SOAPServer("services.soaplite.com", "temper.cgi"),
       "c2f", 37.5,
      action="http://www.soaplite.com/Temperatures")


 # Different action and namespace.
 # Specify handlers=NULL to avoid the additional processing of the
 # SOAPAction string, i.e. the appending of the method name.
 # This kills off all the handlers. If we want to remove only the
 # "action" element, we have to be more explicit.

s1 <- SOAPServer("services.soaplite.com", "interop.cgi")
.SOAP(s1,
      "echoString", "From R",
      action="urn:soapinterop",
      xmlns=c(namesp1="http://soapinterop.org/"),
      handlers =NULL)
}
}
\keyword{interface}
\keyword{connection}
